/*!
 * @pluginjs/font-picker v0.8.16 (https://pluginjs.com)
 * Copyright 2022 Creation Studio Limited
 * Released under the GPL-3.0 License.
 */
import Component from '@pluginjs/component';
import templateEngine from '@pluginjs/template';
import { translateable, themeable, styleable, eventable, stateable, optionable, register } from '@pluginjs/decorator';
import { search } from '@pluginjs/match';
import { isString, isIE, isIE11, isNull, isArray, isPlainObject, isFunction, isEmpty, isObject } from '@pluginjs/is';
import { appendTo, insertBefore, query, parseHTML, detach, prependTo, html, insertAfter, closest, empty } from '@pluginjs/dom';
import { bindEvent, removeEvent } from '@pluginjs/events';
import { addClass, removeClass } from '@pluginjs/classes';
import { each, deepClone, triggerNative } from '@pluginjs/utils';
import Dropdown from '@pluginjs/dropdown';
import { intersectionObserverPolyfill } from '@pluginjs/polyfills';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var namespace = 'fontPicker';
var events = {
  READY: 'ready',
  ENABLE: 'enable',
  DISABLE: 'disable',
  DESTROY: 'destroy',
  SELECT: 'select',
  UNSELECT: 'unselect',
  CHANGE: 'change',
  HIDE: 'hide',
  HIDED: 'hided',
  SHOW: 'show',
  SHOWN: 'shown',
  CLEAR: 'clear',
  FILTER: 'filter'
};
var classes = {
  NAMESPACE: "pj-".concat(namespace),
  ELEMENT: '{namespace}-element',
  TRIGGER: '{namespace}-trigger pj-input',
  LABEL: '{namespace}-label',
  WRAP: '{namespace}',
  SHOW: '{namespace}-show',
  DROPDOWN: '{namespace}-dropdown',
  GROUP: '{namespace}-group',
  GROUPLABEL: '{namespace}-group-label',
  GROUPHIDED: '{namespace}-group-hided',
  MAIN: '{namespace}-main',
  SOURCE: '{namespace}-source',
  SOURCEHIDED: '{namespace}-source-hided',
  ITEM: '{namespace}-item pj-dropdown-item',
  SELECTED: '{namespace}-selected',
  DISABLED: '{namespace}-disabled',
  CLEARABLE: '{namespace}-clearable',
  CLEAR: '{namespace}-clear',
  FILTERABLE: '{namespace}-filterable',
  FILTER: '{namespace}-filter',
  EMPTY: '{namespace}-empty',
  NOTFOUND: '{namespace}-not-found',
  LOADING: '{namespace}-loading',
  ACTIONS: '{namespace}-actions',
  ACTION: '{namespace}-action',
  MANAGE: '{namespace}-manage',
  SWITCHER: '{namespace}-switcher',
  SWITCHERLABEL: '{namespace}-switcher-label',
  SWITCHERDROPDOWN: '{namespace}-switcher-dropdown'
};
var methods = ['set', 'get', 'val', 'clear', 'enable', 'disable', 'destroy'];
var defaults = {
  theme: null,
  source: [{
    name: 'system',
    title: 'System',
    icon: 'pj-icon pj-icon-desktop',
    fonts: {
      'Inherited': ['inherit'],
      'Sans Serif': ['Arial', 'Calibri', 'Century', 'Tahoma', 'Trebuchet', 'Verdana', 'System Sans'],
      Serif: ['Georgia', 'Palatino', 'Times'],
      Monospaced: ['Courier'],
      Chinese: ['简宋', '繁宋', '简黑', '繁黑', '简楷', '繁楷', '仿宋']
    },

    load($item, fontFamily) {
      var fonts = {
        inherit: "inherit",
        Arial: '"Arial", "Helvetica Neue", "Helvetica", "Nimbus Sans L", "Liberation Sans", "Arimo", "sans-serif"',
        Calibri: '"Calibri", "Candara", "Gill Sans", "Gill Sans MT", "DejaVu Sans", "Verdana", "Geneva", "sans-serif"',
        Century: '"Century Gothic", "Apple Gothic", "AppleGothic", "URW Gothic L", "Avantgarde", "DejaVu Sans", "Tahoma", "sans-serif"',
        Tahoma: '"Tahoma", "Lucida Grande", "Lucida Sans Unicode", "Lucida Sans", "DejaVu Sans", "Verdana", "Geneva", "sans-serif"',
        Trebuchet: '"Trebuchet MS", "Futura", "Lucida Grande", "Lucida Sans Unicode", "Lucida Sans", "DejaVu Sans", "Tahoma", "sans-serif"',
        Verdana: '"Verdana", "Geneva", "DejaVu Sans", "sans-serif"',
        'System Sans': '"-apple-system", "BlinkMacSystemFont", "Segoe UI", "Roboto", "Oxygen-Sans", "Ubuntu", "Cantarell", "Helvetica Neue", "sans-serif"',
        // Serif
        Georgia: '"Georgia", "Palatino", "Palatino Linotype", "Palatino LT STD", "URW Palladio L", "Book Antiqua", "DejaVu Serif", "serif"',
        Palatino: '"Palatino", "Palatino Linotype", "Palatino LT STD", "URW Palladio L", "Book Antiqua", "DejaVu Serif", "Georgia", "serif"',
        Times: '"Times New Roman", "TimesNewRoman", "DejaVu Serif", "Liberation Serif", "Tinos", "Times", "serif"',
        // Monospaced
        Courier: '"Courier New", "Courier", "Courier 10 Pitch", "Liberation Mono", "Nimbus Mono L", "Cousine", "monospace"',
        // 宋体 serif
        简宋: '"Songti SC", "STSong", "华文宋体", "宋体", "SimSun", "新宋体", "NSimSun", "AR PL New Sung", "AR PL SungtiL GB", "serif"',
        繁宋: '"LiSong Pro", "Apple LiSung", "新細明體", "PMingLiU", "MingLiU", "AR PL Mingti2L", "TW-Sung", "serif"',
        // 黑体 sans-serif
        简黑: '"Heiti SC", "Microsoft YaHei New", "Microsoft Yahei", "微软雅黑", "SimHei", "黑体", "STHeiti Light", "STXihei", "华文细黑", "STHeiti", "华文黑体", "WenQuanYi Zen Hei", "sans-serif"',
        繁黑: '"Heiti TC", "Microsoft JhengHei", "微軟正黑體", "sans-serif"',
        // 楷体
        简楷: '"Kaiti SC", "KaiTi", "楷体", "STKaiti", "华文楷体", "Kai", "AR PL UKai CN", "serif"',
        繁楷: '"BiauKai", "DFKai-SB", "AR PL KaitiM", "AR PL KaitiM GB", "AR PL UKai HK", "AR PL UKai TW", "TW-Kai", "serif"',
        // 仿宋
        仿宋: '"FangSong", "Fang Song", "仿宋", "STFangSong", "华文仿宋", "serif"'
      };
      $item.style.fontFamily = fonts[fontFamily];
    }

  }],
  placeholder: true,
  responsiveDropdownFull: false,
  clearable: true,
  manage: null,
  multiple: false,
  filterable: true,

  filter(item, query) {
    return search(query, item, {
      diacritics: false,
      punctuation: false,
      case: false,
      whitespaces: false,
      boundaries: false
    });
  },

  keyboard: true,
  dropdown: {
    placement: 'bottom-start' // top

  },
  tooltip: {
    trigger: 'hover'
  },
  templates: {
    dropdown() {
      return '<div class="{classes.DROPDOWN}"><div class="{classes.MAIN}"></div></div>';
    },

    filter() {
      return '<div class="{classes.FILTER}"><input type="text" autocomplete="off" spellcheck="false" placeholder="{placeholder}"></div>';
    },

    switcher() {
      return '<div class="{classes.SWITCHER} {classes.ACTION}"><div class="{classes.SWITCHERLABEL}">{label}</div><div class="{classes.SWITCHERDROPDOWN}"></div></div>';
    },

    switcherLabel() {
      return '<i class="{source.icon}"></i> <span>{source.title}</span>';
    },

    manage() {
      return '<div class="{classes.MANAGE} {classes.ACTION}">{text}</div>';
    },

    label() {
      return '<div class="{classes.LABEL}">{placeholder}</div>';
    },

    source() {
      return '<div class="{classes.SOURCE}" data-name="{source.name}"></div>';
    },

    group() {
      return '<div class="{classes.GROUP}"><div class="{classes.GROUPLABEL}">{group}</div></div>';
    },

    item() {
      return '<div class="{classes.ITEM}" data-source="{source.name}" data-value="{font}">{font}</div>';
    },

    selected() {
      return '<i class="{source.icon}"></i> <span>{value}</span>';
    }

  },

  parse(data) {
    if (isString(data)) {
      try {
        return JSON.parse(data);
      } catch (e) {
        return null;
      }
    }

    return null;
  },

  process(data) {
    if (data && typeof data !== 'undefined' && data.length !== 0) {
      return JSON.stringify(data);
    }

    return '';
  }

};
var translations = {
  en: {
    placeholderText: 'Select Font',
    loadingText: 'loading..',
    notFoundText: 'No fonts found',
    searchText: 'Search',
    manageText: 'Manage',
    swicherText: 'Sources'
  },
  zh: {
    placeholderText: '选择字体',
    loadingText: '加载中..',
    notFoundText: '无匹配字体',
    searchText: '搜索',
    manageText: '管理',
    swicherText: '字体'
  }
};
var dependencies = ['dropdown'];

class Clearable {
  constructor(instance) {
    this.element = appendTo("<i class=\"".concat(instance.classes.CLEAR, "\"></i>"), instance.$trigger);
    addClass(instance.classes.CLEARABLE, instance.$wrap);
    bindEvent(instance.eventName('mousedown'), () => {
      instance.clear();
      return false;
    }, this.element);
  }

  destroy() {
    if (isIE() || isIE11()) {
      this.element.removeNode(true);
    } else {
      this.element.remove();
    }
  }

}

class Filterable {
  constructor(instance) {
    this.instance = instance;
    this.element = insertBefore(templateEngine.render(instance.options.templates.filter.call(instance), {
      classes: instance.classes,
      placeholder: instance.translate('searchText')
    }), instance.$main);
    this.$input = query('input', this.element);
    addClass(instance.classes.FILTERABLE, instance.$wrap);
    bindEvent(instance.selfEventName(events.SHOWN), () => {
      this.$input.focus();
    }, instance.element);
    bindEvent(instance.eventName('input'), () => {
      var value = this.$input.value;
      var DROPDOWN = instance.DROPDOWN;

      if (!value) {
        this.refreshDefault();
      } else {
        this.filter(value);
      }

      if (isNull(DROPDOWN.getHighlightedItem())) {
        DROPDOWN.highlightItem(DROPDOWN.getHighlightableItem());
      }

      if (!DROPDOWN.is('shown')) {
        DROPDOWN.show();
      } else {
        DROPDOWN.update();
      }

      instance.trigger(events.FILTER, value);
    }, this.$input);
    bindEvent(instance.selfEventName(events.HIDE), () => {
      this.$input.value = '';
      this.refreshDefault();
      this.hideNotFound();
    }, instance.element);
  }

  showNotFound() {
    if (this.notfound) {
      return;
    }

    if (!this.$notFound) {
      this.$notFound = parseHTML("<div class=\"".concat(this.instance.classes.NOTFOUND, "\">").concat(this.instance.translate('notFoundText'), "</div>"));
    }

    appendTo(this.$notFound, this.instance.$main);
    this.notfound = true;
  }

  hideNotFound() {
    if (!this.notfound) {
      return;
    }

    if (this.$notFound) {
      detach(this.$notFound);
    }

    this.notfound = false;
  }

  showItem(item) {
    if (item.hided) {
      if (this.instance.DROPDOWN) {
        this.instance.DROPDOWN.showItem(item.$dom);
      }

      item.hided = false;
    }
  }

  hideItem(item) {
    if (typeof item.hided === 'undefined' || item.hided === false) {
      if (this.instance.DROPDOWN) {
        this.instance.DROPDOWN.hideItem(item.$dom);
      }

      item.hided = true;
    }
  }

  showGroup(group) {
    if (group.hided) {
      removeClass(this.instance.classes.GROUPHIDED, group.$dom);
      group.hided = false;
    }
  }

  hideGroup(group) {
    if (typeof group.hided === 'undefined' || group.hided === false) {
      addClass(this.instance.classes.GROUPHIDED, group.$dom);
      group.hided = true;
    }
  }

  filter(query) {
    var {
      filter
    } = this.instance.options;
    var found = 0;
    var source = this.instance.getCurrentSource();
    var items = this.instance.getCurrentSourceItems();

    if (isArray(source.fonts)) {
      each(items, (k, item) => {
        if (filter(item.font, query)) {
          this.showItem(item);
          found++;
        } else {
          this.hideItem(item);
        }
      });
    } else {
      each(items, (name, group) => {
        var groupFound = 0;
        each(group.items, (k, item) => {
          if (filter(item.font, query)) {
            this.showItem(item);
            found++;
            groupFound++;
          } else {
            this.hideItem(item);
          }
        });

        if (groupFound) {
          this.showGroup(group);
        } else {
          this.hideGroup(group);
        }
      });
    }

    if (found) {
      this.hideNotFound();
    } else {
      this.showNotFound();
    }
  }

  refreshDefault() {
    this.hideNotFound();
    this.$input.value = '';
    var source = this.instance.getCurrentSource();
    var items = this.instance.getCurrentSourceItems();

    if (isArray(source.fonts)) {
      each(items, (k, item) => {
        this.showItem(item);
      });
    } else {
      each(items, (name, group) => {
        each(group.items, (k, item) => {
          this.showItem(item);
        });
        this.showGroup(group);
      });
    }
  }

}

class Manage {
  constructor(instance) {
    this.element = appendTo(templateEngine.render(instance.options.templates.manage.call(instance), {
      classes: instance.classes,
      text: instance.translate('manageText')
    }), instance.getActions());
    bindEvent(instance.eventName('click'), () => {
      instance.manage();
    }, this.element);
  }

}

class Switcher {
  constructor(instance) {
    this.instance = instance;
    this.element = prependTo(templateEngine.render(instance.options.templates.switcher.call(instance), {
      classes: instance.classes,
      label: this.getLabel(instance.current)
    }), instance.getActions());
    this.$label = query(".".concat(instance.classes.SWITCHERLABEL), this.element);
    this.DROPDOWN = Dropdown.of(this.$label, {
      reference: this.element,
      target: query(".".concat(instance.classes.SWITCHERDROPDOWN), this.element),
      placement: 'top-start',
      value: instance.current.name,
      data: instance.getSources().map(source => {
        return {
          value: source.name,
          label: this.getLabel(source)
        };
      }),
      keyboard: instance.options.keyboard,
      onChange: value => {
        instance.switchSource(value);
        var source = instance.getSource(value);
        html(this.getLabel(source), this.$label);
      }
    });
  }

  getLabel(source) {
    return templateEngine.render(this.instance.options.templates.switcherLabel.call(this.instance), {
      classes: this.instance.classes,
      source
    });
  }

  destroy() {
    this.DROPDOWN.destroy();

    if (isIE() || isIE11()) {
      this.element.removeNode(true);
    } else {
      this.element.remove();
    }
  }

}

class Loading {
  constructor(instance) {
    this.instance = instance;
  }

  show() {
    if (!this.element) {
      this.element = parseHTML("<div class=\"".concat(this.instance.classes.LOADING, "\">").concat(this.instance.translate('loadingText'), "</div>"));
    }

    appendTo(this.element, this.instance.$main);
  }

  hide() {
    if (this.element) {
      detach(this.element);
    }
  }

}

intersectionObserverPolyfill();
class FontLoader {
  constructor(instance) {
    this.instance = instance;
    this.createObserver();
  }

  createObserver() {
    this.observer = new IntersectionObserver((changes, observer) => {
      changes.forEach(change => {
        if (change.isIntersecting) {
          var $item = change.target;
          observer.unobserve($item);
          this.load($item);
        }
      });
    }, {
      root: this.instance.$main
    });
  }

  observe(items) {
    each(items, (k, item) => {
      if (item.$dom.dataset.loaded !== true) {
        this.observer.observe(item.$dom);
      }
    });
  }

  load($item) {
    $item.dataset.loaded = true;
    var source = this.instance.getSource($item.dataset.source);
    source.load($item, $item.dataset.value, $item.innerText);
  }

  reset() {
    this.observer.disconnect();
    this.createObserver();
  }

}

var _dec, _dec2, _dec3, _dec4, _dec5, _dec6, _dec7, _class;
var FontPicker = (_dec = translateable(translations), _dec2 = themeable(), _dec3 = styleable(classes), _dec4 = eventable(events), _dec5 = stateable(), _dec6 = optionable(defaults, true), _dec7 = register(namespace, {
  methods: methods,
  dependencies: dependencies
}), _dec(_class = _dec2(_class = _dec3(_class = _dec4(_class = _dec5(_class = _dec6(_class = _dec7(_class = class FontPicker extends Component {
  constructor(element) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    super(element);
    this.setupOptions(options);
    this.setupClasses();
    this.setupStates();
    this.setupI18n();
    this.initialize();
  }

  initialize() {
    this.selected = null;
    this.current = null;
    this.placeholder = this.element.getAttribute('placeholder');

    if (!this.placeholder && this.options.placeholder) {
      if (this.options.placeholder === true) {
        this.placeholder = this.translate('placeholderText');
      } else {
        this.placeholder = this.options.placeholder;
      }
    }

    addClass(this.classes.ELEMENT, this.element);
    this.$wrap = insertAfter("<div class=\"".concat(this.classes.WRAP, "\"></div>"), this.element);

    if (this.options.theme) {
      addClass(this.getThemeClass(), this.$wrap);
    }

    this.$trigger = appendTo("<div class=\"".concat(this.classes.TRIGGER, "\"></div>"), this.$wrap);
    this.$label = appendTo(templateEngine.render(this.options.templates.label(), {
      classes: this.classes,
      placeholder: this.placeholder
    }), this.$trigger);
    this.bind();
    this.setupDropdown(this.options.dropdown);
    this.initData();

    if (this.options.clearable) {
      this.CLEARABLE = new Clearable(this);
    }

    this.LOADING = new Loading(this);

    if (this.element.disabled || this.options.disabled) {
      this.disable();
    }

    this.enter('initialized');
    this.trigger(events.READY);
  }

  setupDropdown(options) {
    this.$dropdown = appendTo(templateEngine.render(this.options.templates.dropdown.call(this), {
      classes: this.classes
    }), this.$wrap);
    this.$main = query(".".concat(this.classes.MAIN), this.$dropdown);
    this.DROPDOWN = Dropdown.of(this.$trigger, _objectSpread2(_objectSpread2({}, options), {}, {
      target: this.$dropdown,
      responsiveFull: this.options.responsiveDropdownFull,
      keyboard: this.options.keyboard ? this.$wrap : false,
      classes: {
        PLACEMENT: "".concat(this.classes.NAMESPACE, "-on-{placement}")
      },
      onShow: () => {
        if (this.is('loading')) {
          this.LOADING.show();
        }

        if (!this.is('builded')) {
          this.buildDropdown();
        }

        addClass(this.classes.SHOW, this.$wrap);
        this.trigger(events.SHOW);
      },
      onShown: () => {
        this.trigger(events.SHOWN);
        this.enter('shown');
      },
      onHide: () => {
        this.trigger(events.HIDE);
        this.leave('shown');
      },
      onHided: () => {
        removeClass(this.classes.SHOW, this.$wrap);
        this.trigger(events.HIDED);
      },
      onSelect: $item => {
        var $source = closest(".".concat(this.classes.SOURCE), $item);
        var source = this.getSource($source.dataset.name);
        this.set({
          source: source.name,
          font: $item.dataset.value
        });
      }
    }));
  }

  bind() {} // eslint-disable-line


  unbind() {
    removeEvent(this.eventName(), this.element);
  }

  buildDropdown() {
    if (this.options.manage) {
      this.MANAGE = new Manage(this);
    }

    if (!this.FONTLOADER) {
      this.FONTLOADER = new FontLoader(this);
    }

    if (this.data) {
      this.buildDropdownContent();
    }

    this.enter('builded');
  }

  initData() {
    if (isArray(this.options.source) || isPlainObject(this.options.source)) {
      this.resolveData(this.options.source);
    } else if (isFunction(this.options.source)) {
      this.enter('loading');
      this.options.source.call(this, this.resolveData.bind(this));
    }
  }

  resolveData(data) {
    this.data = deepClone(data);

    if (!isArray(this.data)) {
      this.data = [this.data];
    }

    if (this.data.length < 1) {
      return;
    }

    var value = this.element.value;

    if (!isEmpty(value)) {
      this.set(this.options.parse.call(this, value), false);
    }

    if (isNull(this.current)) {
      this.setCurrentSource(this.data[0].name);
    }

    if (this.is('loading')) {
      if (this.LOADING) {
        this.LOADING.hide();
      }

      this.leave('loading');
    }

    if (this.is('builded') || this.is('shown')) {
      this.buildDropdownContent();
    }
  }

  set(item) {
    var trigger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

    if (item !== this.selected) {
      if (this.selected) {
        if (this.DROPDOWN) {
          this.DROPDOWN.unselectByValue(this.getItemValue(this.selected), false);
        }
      }

      if (isNull(item)) {
        this.setPlaceholderLabel();
        removeClass(this.classes.SELECTED, this.$wrap);

        if (trigger) {
          this.trigger(events.CLEAR);
        }
      } else {
        this.setSelectedLabel(item);
        addClass(this.classes.SELECTED, this.$wrap);

        if (trigger) {
          this.trigger(events.SELECT, item);
        }
      }

      this.selected = item;
      var value;

      if (item) {
        if (item.source) {
          this.setCurrentSource(item.source);

          if (this.SWITCHER) {
            this.SWITCHER.DROPDOWN.selectByValue(this.current.name, false);
            html(this.SWITCHER.getLabel(this.getSource(this.current.name)), this.SWITCHER.$label);

            if (this.current.__dom) {
              removeClass(this.classes.SOURCEHIDED, this.current.__dom);
            } else {
              this.buildSource(this.getCurrentSource());
            }

            this.selectForDropdown();
          }
        }

        value = this.options.process.call(this, item);
      } else {
        value = '';
      }

      if (value !== this.element.value) {
        this.element.value = value;

        if (trigger) {
          this.trigger(events.CHANGE, value);
          triggerNative(this.element, 'change');
        }
      }
    }
  }

  val(value) {
    if (typeof value === 'undefined') {
      return this.options.process.call(this, this.get());
    }

    return this.set(this.options.parse.call(this, value));
  }

  get() {
    if (this.selected) {
      return this.selected;
    }

    return null;
  }

  clear() {
    this.set(null);
  }

  getItemLabel(item) {
    return templateEngine.render(this.options.templates.selected.call(this), {
      classes: this.classes,
      value: item.font,
      source: this.getSource(item.source)
    });
  }

  getCurrentSource() {
    return this.current;
  }

  setCurrentSource(name) {
    var source = this.getSource(name);

    if (source) {
      this.current = source;
    } else {
      this.current = this.data.length > 0 ? this.data[0] : null;
    }
  }

  switchSource(name) {
    if (name !== this.current.name) {
      if (this.FILTERABLE) {
        this.FILTERABLE.refreshDefault();
      }

      addClass(this.classes.SOURCEHIDED, this.current.__dom);
      var source = this.getSource(name);

      if (!source.__dom) {
        source.__dom = this.buildSource(source);
      } else {
        removeClass(this.classes.SOURCEHIDED, source.__dom);
      }

      this.$main.scrollTop = 0;
      this.setCurrentSource(name);
    }
  }

  getSource(name) {
    return this.data.find(source => source.name === name);
  }

  getSources() {
    return this.data;
  }

  getCurrentSourceItems() {
    if (this.current && this.current.__items) {
      return this.current.__items;
    }

    return [];
  }

  getItemValue(item) {
    return item.font;
  }

  setLabel(label) {
    html(label, this.$label);
  }

  setPlaceholderLabel() {
    this.setLabel(this.placeholder);
    this.$label.style.fontFamily = null;
  }

  setSelectedLabel(item) {
    this.setLabel(this.getItemLabel(item));
    var source = this.getSource(item.source);

    if (source) {
      source.load(this.$label, item.font);
    }
  }

  selectForDropdown() {
    if (this.selected) {
      this.DROPDOWN.selectByValue(this.getItemValue(this.selected), false);
    }
  }

  buildDropdownContent() {
    if (this.SWITCHER && query('.pj-fontPicker-switcher', this.getActions())) {
      if (isIE() || isIE11()) {
        query('.pj-fontPicker-switcher', this.getActions()).removeNode(true);
      } else {
        query('.pj-fontPicker-switcher', this.getActions()).remove();
      }
    }

    if (this.data.length > 1) {
      this.SWITCHER = new Switcher(this);
    }

    if (this.options.filterable && !this.FILTERABLE) this.FILTERABLE = new Filterable(this);
    this.$main.innerHTML = "";
    this.buildSource(this.getCurrentSource());
    this.selectForDropdown();
  }

  buildSource(source) {
    if (!this.sourceTemplate) {
      this.sourceTemplate = templateEngine.compile(this.options.templates.source.call(this));
    }

    var $source = parseHTML(this.sourceTemplate({
      classes: this.classes,
      source
    }));
    source.__dom = $source;
    source.__items = {};

    if (isArray(source.fonts)) {
      source.fonts.forEach(font => {
        $source.appendChild(this.buildItem(source, font));
      });
    } else if (isObject(source.fonts)) {
      each(source.fonts, (category, fonts) => {
        $source.appendChild(this.buildGroup(source, category, fonts));
      });
    }

    this.$main.appendChild($source);

    if (isArray(source.fonts)) {
      this.FONTLOADER.observe(source.__items);
    } else if (isObject(source.fonts)) {
      each(source.__items, (k, group) => {
        this.FONTLOADER.observe(group.items);
      });
    }

    return $source;
  }

  buildGroup(source, group, fonts) {
    if (!this.groupTemplate) {
      this.groupTemplate = templateEngine.compile(this.options.templates.group.call(this));
    }

    var $group = parseHTML(this.groupTemplate({
      classes: this.classes,
      group
    }));
    source.__items[group] = {
      group,
      items: {},
      $dom: $group
    };
    fonts.forEach(font => {
      $group.appendChild(this.buildItem(source, font, group));
    });
    return $group;
  }

  buildItem(source, font, group) {
    if (!this.itemTemplate) {
      this.itemTemplate = templateEngine.compile(this.options.templates.item.call(this));
    }

    var $item = parseHTML(this.itemTemplate({
      classes: this.classes,
      font,
      source
    }));

    if (group) {
      source.__items[group].items[font] = {
        font,
        $dom: $item
      };
    } else {
      source.__items[font] = {
        font,
        $dom: $item
      };
    }

    return $item;
  }

  getActions() {
    if (!this.$actions) {
      this.$actions = insertAfter("<div class=\"".concat(this.classes.ACTIONS, "\"></div>"), this.$main);
    }

    return this.$actions;
  }

  manage() {
    if (isFunction(this.options.manage)) {
      this.options.manage.call(this, this.resolveManage.bind(this));
    }
  }

  resolveManage(data) {
    if (data) {
      if (this.FILTERABLE) {
        this.FILTERABLE.refreshDefault();
      }

      empty(this.$main);

      if (this.SWITCHER) {
        this.SWITCHER.destroy();
        this.SWITCHER = null;
      }

      this.resolveData(data);
    }
  }

  enable() {
    if (this.is('disabled')) {
      this.DROPDOWN.enable();
      removeClass(this.classes.DISABLED, this.$wrap);
      this.leave('disabled');
    }

    this.trigger(events.ENABLE);
  }

  disable() {
    if (!this.is('disabled')) {
      this.DROPDOWN.disable();
      addClass(this.classes.DISABLED, this.$wrap);
      this.enter('disabled');
    }

    this.trigger(events.DISABLE);
  }

  destroy() {
    if (this.is('initialized')) {
      this.unbind();

      if (this.CLEARABLE) {
        this.CLEARABLE.destroy();
      }

      if (this.options.theme) {
        removeClass(this.getThemeClass(), this.$wrap);
      }

      if (isIE() || isIE11()) {
        this.$wrap.removeNode(true);
      } else {
        this.$wrap.remove();
      }

      removeClass(this.classes.ELEMENT, this.element);
      this.leave('initialized');
    }

    this.trigger(events.DESTROY);
    super.destroy();
  }

}) || _class) || _class) || _class) || _class) || _class) || _class) || _class);

export { FontPicker as default };
